package Export;

import Topology.*;
import Graph.*;
import Model.ModelConstants;
import Util.*;

import java.io.*;
import java.util.Arrays;

/**
 * <code>Export.GraphMLExport</code> provides functionality to export a
 * topology into a GraphML format file. See <a
 * href="http://graphml.graphdrawing.org">http://graphml.graphdrawing.org</a>
 * for a description of this file format.<br/>
 * <br/>
 * Note: This file is a quick adaptation from BriteExport.java and might not
 * support all of BRITE's functionality. Responsible for any bugs contained
 * herein is Frank Zdarsky (zdarsky@informatik.uni-kl.de).
 */
public class GraphMLExport {

	private BufferedWriter bw;

	private Graph g;

	private String modelStr;

	/**
	 * Class Constructor: Returns a GraphMLExport object which your code may
	 * keep around. Does not actually write the topology to the file. You must
	 * explicitly call the <code>export()</code> method of this object in
	 * order to write to the file.
	 * 
	 * @param t        the topology object to export
	 * @param outFile  the destination file to write the topology to.
	 */
	public GraphMLExport(Topology t, File outFile) {
		try {
			bw = new BufferedWriter(new FileWriter(outFile));
		} catch (IOException e) {
			Util.ERR(" Error creating BufferedWriter in GraphMLExport: " + e);
		}
		g = t.getGraph();
		modelStr = t.getModel().toString();
	}

	/**
	 * Writes the contents of the topolgy in the GraphML format to the
	 * destination file specified in the constructor.
	 */
	public void export() {
		Util.MSG("Exporting to GraphML...");
		try {
			// output header
			bw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			bw.newLine();
			bw.write("<?meta name=\"GENERATOR\" content=\"BRITE\"?>");
			bw.newLine();
			bw.write("<!-- " + modelStr.trim() + " -->");
			bw.newLine();
			bw.write("<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\" "
			    + "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
			    + "xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns "
			    + "http://graphml.graphdrawing.org/xmlns/1.0/graphml.xsd\">");
			bw.newLine();
			bw.write("<graph id=\"G\" edgedefault=\"directed\">");
			bw.newLine();

			// output nodes
			Node[] nodes = g.getNodesArray();
			Arrays.sort(nodes, Node.IDcomparator);

			int min_id = nodes[0].getID();

			for (int i = 0; i < nodes.length; ++i) {
				Node n = nodes[i];
				int x = (int) ((NodeConf) n.getNodeConf()).getX();
				int y = (int) ((NodeConf) n.getNodeConf()).getY();
				int nodeID = n.getID();

				int asid = -1;
				String type = "";
				if (n.getNodeConf() instanceof RouterNodeConf) {
					asid = ((RouterNodeConf) n.getNodeConf()).getCorrAS();
					int specificNodeType = ((RouterNodeConf) n.getNodeConf())
							.getType();
					if (specificNodeType == ModelConstants.RT_LEAF)
						type = "RT_LEAF";
					else if (specificNodeType == ModelConstants.RT_BORDER)
						type = "RT_BORDER";
					else if (specificNodeType == ModelConstants.RT_STUB)
						type = "RT_STUB";
					else if (specificNodeType == ModelConstants.RT_BACKBONE)
						type = "RT_BACKBONE";
					else
						type = "RT_NODE";
				} else if (n.getNodeConf() instanceof ASNodeConf) {
					asid = nodeID;
					int specificNodeType = ((ASNodeConf) n.getNodeConf())
							.getType();
					if (specificNodeType == ModelConstants.AS_LEAF)
						type = "AS_LEAF";
					else if (specificNodeType == ModelConstants.AS_BORDER)
						type = "AS_BORDER";
					else if (specificNodeType == ModelConstants.AS_STUB)
						type = "AS_STUB";
					else if (specificNodeType == ModelConstants.AS_BACKBONE)
						type = "AS_BACKBONE";
					else
						type = "AS_NODE";
				}

				bw.write("    <node id=\"" + (nodeID - min_id) + "\"");
				bw.write(" x=\"" + x + "\" y=\"" + y + "\"");
				if (type != null) {
					bw.write(" type=\"" + type + "\" asid=\"" + asid + "\"");
				}
				bw.write("/>");
				bw.newLine();
			}

			// output edges
			Edge[] edges = g.getEdgesArray();
			Arrays.sort(edges, Edge.IDcomparator);
			for (int i = 0; i < edges.length; ++i) {
				Edge e = (Edge) edges[i];
				Node src = e.getSrc();
				Node dst = e.getDst();

				if (e.getDirection() == GraphConstants.DIRECTED) {
					bw.write("    <edge source=\"" + (src.getID() - min_id)
							+ "\" target=\"" + (dst.getID() - min_id) + "\"/>");
					bw.newLine();
				} else {
					bw.write("    <edge source=\"" + (src.getID() - min_id)
							+ "\" target=\"" + (dst.getID() - min_id) + "\"/>");
					bw.newLine();
					bw.write("    <edge source=\"" + (dst.getID() - min_id)
							+ "\" target=\"" + (src.getID() - min_id) + "\"/>");
					bw.newLine();
				}
			}

			// output trailer
			bw.write("  </graph>");
			bw.newLine();
			bw.write("</graphml>");
			bw.newLine();
			bw.close();
		} catch (Exception e) {
			System.out.println("[BRITE ERROR]: Error exporting to file. " + e);
			e.printStackTrace();
			System.exit(0);
		}
		Util.MSG("... DONE.");
	}
}
